#ifndef  ASSM
#define ASSM 101

#include "String.h"
#include "structur.h"
#include "Sym_Lit.h"
#include "OPTable.h"
#include "Obj.h"

class CAssemble
{
private:
	List a;
	FILE *fptr;             //Pointer to file to be assembled ( input )
	FILE *fpList;			//Pointer to List file ( output )
	int LC;					//Location counter
	int Done;				// Indicates if the END pseudo op has been encountered
	int idx;				//index of MOT entry
	CSymLit Sym;			//The Symbol table
	CObjFile *m_ObjFile;	//pointer to CObjFile class
	C_OP_Table Tables;		//The Machine and Pseudo op tables
	CString String;  		//holds one line of code at a time
	CString Label; 			//holds the label field
	CString mne_op;			//holds the mnemonic op code
	CString ListFile, ObjectFile, Name;
	CString Operand1, Operand2;	//Holds the operands
	int Ptr;				// 1-BYTE, 2-WORD, 4-DWORD, 0-NOT SET
	CString CurrentSegment; //Label of current Segment Register
	CString CurrentProcedure; //Label of the current Procedure
	BYTE ProcedureType;		//Procedure Type F-Far, N-Near
	CString CS, DS, ES, SS;	// Holds the 'ASSUMED' labels
	BYTE MachineCode[6];	//Machine code
	long Read;				//No of characters processed in variable str
	long LineNo;			//Line no
	int LabelLength;		//Holds the maximum length of label - 17
	int GetMOT(CString &);  //returns the index of a machine op
	int GetNextMOT(CString &, int start);     //returns the index of the next machine op with the same mnemonic
	int GetPOT(CString &);	//returns the index of a pseudo op
	int IsDelimiter(char);  //Deremines if a character is a delimiting character
	int GetNextToken(CString &String,CString &Token, int start);
							//Retrives the next Token and returns its the number of characters traversed
	int EvaluateExpression(CString &, int & error); //Evaluates an expression for the EQU handler

	int GetOperands();		//Retrieve the operands if any from str.
	int CheckType (MOT *pmot,int pass); //Checks if instruction is of type "type"
	int CheckType0(MOT *pmot,int pass);
	int CheckType1(MOT *pmot,int pass);
	int CheckType2(MOT *pmot,int pass); //Checks if instruction is of type 2
	int CheckType3(MOT *pmot,int pass);
	int CheckType4(MOT *pmot,int pass);
	int CheckType5(MOT *pmot,int pass);
	int CheckType6(MOT *pmot,int pass);
	int CheckType7(MOT *pmot,int pass);
	int CheckType8(MOT *pmot,int pass);
	BYTE CAssemble::CheckIndices(CString &Part1, CString &Part2, int &disp, int pass, int & error);
	int IsNear(CString & Operand);
	int IsAddress(CString & Operand);
	int IsDirectAddress(CString & Operand);
	int GetAddress(CString Operand,int &error);
	int IsImmediate(CString & Operand);
	BYTE GetHigh(int value);
	BYTE GetLow(int value);
	int CanBeLabel(CString & Operand);  //CouldBeLabel
	CString StripBraces(CString & );
	void Error(CString &Operand);
	void Error(char *Operand);
	void Warning(CString &Operand);
	void Warning(char *Operand);
	void WriteSymbolTable(FILE *fp);
	//#################### Pseudo op handlers  ####################
	int HandlePseudoOP(int op_id, int pass);//Handles all Pseudo ops for Pass1 & 2
	int DataDefinition(int pass, int size);
	int ASSUME_2_pop();
	int EQU_1_pop();
	int EXTRN_1_pop();
	int PUBLIC_1_pop();
	int SEGMENT_1_pop();
	int SEGMENT_2_pop();
	int ENDS_pop();
	int END_1_pop();        //Handles the END Pseudo op (Pass 1)
	int END_2_pop();        //Handles the END Pseudo op (Pass 2)
	int GLOBAL_1_pop();
	int PROC_pop( int pass );
	int ENDP_pop( );
public:
	int Warnings,Errors;	//No of Warnings and errors found

	CAssemble();            //Default constructor
	CAssemble(char *);		//Another constructor (this is the one that I have actually used in my program.)
	~CAssemble();			//The Destructor
	int Pass1();			//First of the two pass assembler
	int Pass2();            //Second of the two pass assembler
};
#endif